# YML 2.1.9 language definition

# written by VB.

import re
from pyPEG import keyword, _and, _not

# pyPEG:
#
#   basestring:     terminal symbol (characters)
#   keyword:        terminal symbol (keyword)
#   matchobj:       terminal symbols (regex, use for scanning symbols)
#   function:       named non-terminal symbol, recursive definition
#                   if you don't want naming in output, precede name with an underscore
#   tuple:          production sequence
#   integer:        count in production sequence:
#                    0: following element is optional
#                   -1: following element can be omitted or repeated endless
#                   -2: following element is required and can be repeated endless
#   list:           options, choose one of them
#   _not:           next element in production sequence is matched only if this would not
#   _and:           next element in production sequence is matched only if this would, too

def listing(x):     return x, -1, (",", x)
r = re.compile

def comment(): return [r(r"//.*"), r(r"/\*.*?\*/", re.S)]
symbol = r(r"[a-zA-Z_](\w|:)*")
pointer = r(r"\*[a-zA-Z_]\w*")
macro = r(r"\%[a-zA-Z_]\w*")
reference = r(r"\&[a-zA-Z_]\w*")
xmlSymbol = r(r"[a-zA-Z][\-:a-zA-Z0-9]*")
aliasSymbol = r(r"\-|[a-zA-Z][\-:a-zA-Z0-9]*")
filename = r(r"[^\s;]+")
literal = r(r"""-?\d+\.\d*|-?\.\d+|-?\d+|".*?"|'.*?'""")
ws = r(r"\s+")
def pyExp():        return "!", r(r"(!=|\\!|[^!])+"), "!"
value = [literal, pyExp]

def tagQuote():     return r(r"\].*|\<.*?\>")
def lineQuote():    return r(r"\|.*")
def quote():        return [r(r"\d*>.*"), (literal, 0, ";")]
def parm():         return [([xmlSymbol, pyExp, pointer, macro], "=", [value, pointer]), value, pointer]
def _func():        return [symbol, pointer, reference], [("(", listing(parm), ")"), (0, listing(parm))]
def pythonCall():   return keyword("python"), _func, ";"
def declParm():     return [xmlSymbol, pointer, macro], 0, ("=", literal)
def alias():        return keyword("alias"), aliasSymbol
def descend():      return r(r"\+[a-zA-Z][\-:a-zA-Z]*")
def base():         return keyword("is"), symbol
def decl():         return symbol, 0, base, -1, descend, 0, ("(", 0, listing(declParm), ")"), 0, alias, 0, content
def python():       return [r(r"!!.*?!!", re.S), r(r"!.*")]
def operator():     return keyword("define"), keyword("operator"), literal, keyword("as"), r(r".*")
def in_ns():        return keyword("in"), xmlSymbol, [_decl, ("{", -2, _decl, "}")]
def _decl():        return keyword("decl"), listing(decl), ";"
def textsection():  return r(r'(\|\|(\>*).*?\|\|(\>*))|(""".*?""")|(\>\>.*?\>\>)', re.S)
def _inner():       return [include, textsection, pythonCall, _cmd, quote, lineQuote, tagQuote, pyExp]
def parentheses():  return "(", 0, funclist, ")"
def content():      return [(parentheses, 0, [_inner, ("{", -1, _inner, "}")]), [_inner, ("{", -1, _inner, "}")]]
def func():         return _func, 0, content
def funclist():     return listing(func)
def _cmd():         return funclist, 0, ";"
def reverse():      return keyword("reverse")
def ktext():        return keyword("text")
def kxml():         return keyword("xml")
def include():      return keyword("include"), 0, reverse, 0, [ktext, kxml], filename, 0, ";"
def ymlCStyle():    return -1, [_decl, in_ns, include, python, operator, tagQuote, _cmd]
